@@ -148,10 +148,8 @@ static int qca_read_fw_build_info(struct hci_dev *hdev)
 	}
 
 	build_label = kstrndup(&edl->data[1], build_lbl_len, GFP_KERNEL);
-	if (!build_label) {
-		err = -ENOMEM;
+	if (!build_label)
 		goto out;
-	}
 
 	hci_set_fw_info(hdev, "%s", build_label);
 
@@ -289,39 +287,6 @@ int qca_send_pre_shutdown_cmd(struct hci_dev *hdev)
 }
 EXPORT_SYMBOL_GPL(qca_send_pre_shutdown_cmd);
 
-static bool qca_filename_has_extension(const char *filename)
-{
-	const char *suffix = strrchr(filename, '.');
-
-	/* File extensions require a dot, but not as the first or last character */
-	if (!suffix || suffix == filename || *(suffix + 1) == '\0')
-		return 0;
-
-	/* Avoid matching directories with names that look like files with extensions */
-	return !strchr(suffix, '/');
-}
-
-static bool qca_get_alt_nvm_file(char *filename, size_t max_size)
-{
-	char fwname[64];
-	const char *suffix;
-
-	/* nvm file name has an extension, replace with .bin */
-	if (qca_filename_has_extension(filename)) {
-		suffix = strrchr(filename, '.');
-		strscpy(fwname, filename, suffix - filename + 1);
-		snprintf(fwname + (suffix - filename),
-		       sizeof(fwname) - (suffix - filename), ".bin");
-		/* If nvm file is already the default one, return false to skip the retry. */
-		if (strcmp(fwname, filename) == 0)
-			return false;
-
-		snprintf(filename, max_size, "%s", fwname);
-		return true;
-	}
-	return false;
-}
-
 static int qca_tlv_check_data(struct hci_dev *hdev,
 			       struct qca_fw_config *config,
 			       u8 *fw_data, size_t fw_size,
@@ -619,19 +584,6 @@ static int qca_download_firmware(struct hci_dev *hdev,
 					   config->fwname, ret);
 				return ret;
 			}
-		}
-		/* If the board-specific file is missing, try loading the default
-		 * one, unless that was attempted already.
-		 */
-		else if (config->type == TLV_TYPE_NVM &&
-			 qca_get_alt_nvm_file(config->fwname, sizeof(config->fwname))) {
-			bt_dev_info(hdev, "QCA Downloading %s", config->fwname);
-			ret = reject_firmware(&fw, config->fwname, &hdev->dev);
-			if (ret) {
-				bt_dev_err(hdev, "QCA Failed to request file: %s (%d)",
-					   config->fwname, ret);
-				return ret;
-			}
 		} else {
 			bt_dev_err(hdev, "QCA Failed to request file: %s (%d)",
 				   config->fwname, ret);
@@ -768,38 +720,21 @@ static int qca_check_bdaddr(struct hci_dev *hdev, const struct qca_fw_config *co
 	return 0;
 }
 
-static void qca_get_nvm_name_by_board(char *fwname, size_t max_size,
-		const char *stem, enum qca_btsoc_type soc_type,
+static void qca_generate_hsp_nvm_name(char *fwname, size_t max_size,
 		struct qca_btsoc_version ver, u8 rom_ver, u16 bid)
 {
 	const char *variant;
-	const char *prefix;
 
-	/* Set the default value to variant and prefix */
-	variant = "";
-	prefix = "b";
+	/* hsp gf chip */
+	if ((le32_to_cpu(ver.soc_id) & QCA_HSP_GF_SOC_MASK) == QCA_HSP_GF_SOC_ID)
+		variant = "g";
+	else
+		variant = "";
 
-	if (soc_type == QCA_QCA2066)
-		prefix = "";
-
-	if (soc_type == QCA_WCN6855 || soc_type == QCA_QCA2066) {
-		/* If the chip is manufactured by GlobalFoundries */
-		if ((le32_to_cpu(ver.soc_id) & QCA_HSP_GF_SOC_MASK) == QCA_HSP_GF_SOC_ID)
-			variant = "g";
-	}
-
-	if (rom_ver != 0) {
-		if (bid == 0x0 || bid == 0xffff)
-			snprintf(fwname, max_size, "/*(DEBLOBBED)*/", stem, rom_ver, variant);
-		else
-			snprintf(fwname, max_size, "/*(DEBLOBBED)*/", stem, rom_ver,
-						variant, prefix, bid);
-	} else {
-		if (bid == 0x0 || bid == 0xffff)
-			snprintf(fwname, max_size, "/*(DEBLOBBED)*/", stem, variant);
-		else
-			snprintf(fwname, max_size, "/*(DEBLOBBED)*/", stem, variant, prefix, bid);
-	}
+	if (bid == 0x0)
+		snprintf(fwname, max_size, "/*(DEBLOBBED)*/", rom_ver, variant);
+	else
+		snprintf(fwname, max_size, "/*(DEBLOBBED)*/", rom_ver, variant, bid);
 }
 
 int qca_uart_setup(struct hci_dev *hdev, uint8_t baudrate,
@@ -807,7 +742,6 @@ int qca_uart_setup(struct hci_dev *hdev, uint8_t baudrate,
 		   const char *firmware_name)
 {
 	struct qca_fw_config config = {};
-	const char *variant = "";
 	int err;
 	u8 rom_ver = 0;
 	u32 soc_ver;
@@ -883,39 +817,34 @@ int qca_uart_setup(struct hci_dev *hdev, uint8_t baudrate,
 	/* Give the controller some time to get ready to receive the NVM */
 	msleep(10);
 
-	if (soc_type == QCA_QCA2066 || soc_type == QCA_WCN7850)
+	if (soc_type == QCA_QCA2066)
 		qca_read_fw_board_id(hdev, &boardid);
 
 	/* Download NVM configuration */
 	config.type = TLV_TYPE_NVM;
 	if (firmware_name) {
-		/* The firmware name has an extension, use it directly */
-		if (qca_filename_has_extension(firmware_name)) {
-			snprintf(config.fwname, sizeof(config.fwname), "/*(DEBLOBBED)*/", firmware_name);
-		} else {
-			qca_read_fw_board_id(hdev, &boardid);
-			qca_get_nvm_name_by_board(config.fwname, sizeof(config.fwname),
-				 firmware_name, soc_type, ver, 0, boardid);
-		}
+		snprintf(config.fwname, sizeof(config.fwname),
+			 "/*(DEBLOBBED)*/", firmware_name);
 	} else {
 		switch (soc_type) {
 		case QCA_WCN3990:
 		case QCA_WCN3991:
 		case QCA_WCN3998:
-			if (le32_to_cpu(ver.soc_id) == QCA_WCN3991_SOC_ID)
-				variant = "u";
-
-			snprintf(config.fwname, sizeof(config.fwname),
-				 "/*(DEBLOBBED)*/", rom_ver, variant);
+			if (le32_to_cpu(ver.soc_id) == QCA_WCN3991_SOC_ID) {
+				snprintf(config.fwname, sizeof(config.fwname),
+					 "/*(DEBLOBBED)*/", rom_ver);
+			} else {
+				snprintf(config.fwname, sizeof(config.fwname),
+					 "/*(DEBLOBBED)*/", rom_ver);
+			}
 			break;
 		case QCA_WCN3988:
 			snprintf(config.fwname, sizeof(config.fwname),
 				 "/*(DEBLOBBED)*/", rom_ver);
 			break;
 		case QCA_QCA2066:
-			qca_get_nvm_name_by_board(config.fwname,
-				sizeof(config.fwname), "hpnv", soc_type, ver,
-				rom_ver, boardid);
+			qca_generate_hsp_nvm_name(config.fwname,
+				sizeof(config.fwname), ver, rom_ver, boardid);
 			break;
 		case QCA_QCA6390:
 			snprintf(config.fwname, sizeof(config.fwname),
@@ -926,14 +855,14 @@ int qca_uart_setup(struct hci_dev *hdev, uint8_t baudrate,
 				 "/*(DEBLOBBED)*/", rom_ver);
 			break;
 		case QCA_WCN6855:
-			qca_read_fw_board_id(hdev, &boardid);
-			qca_get_nvm_name_by_board(config.fwname, sizeof(config.fwname),
-						  "hpnv", soc_type, ver, rom_ver, boardid);
+			snprintf(config.fwname, sizeof(config.fwname),
+				 "/*(DEBLOBBED)*/", rom_ver);
 			break;
 		case QCA_WCN7850:
-			qca_get_nvm_name_by_board(config.fwname, sizeof(config.fwname),
-				 "hmtnv", soc_type, ver, rom_ver, boardid);
+			snprintf(config.fwname, sizeof(config.fwname),
+				 "/*(DEBLOBBED)*/", rom_ver);
 			break;
+
 		default:
 			snprintf(config.fwname, sizeof(config.fwname),
 				 "/*(DEBLOBBED)*/", soc_ver);
